@@ -1,18 +1,14 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure Tor.
-"""
-
-import json
+"""FreedomBox app to configure Tor."""
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import action_utils, actions
+from plinth import action_utils
 from plinth import app as app_module
 from plinth import cfg, menu
 from plinth.daemon import (Daemon, app_is_running, diagnose_netcat,
                            diagnose_port_listening)
-from plinth.modules.apache.components import diagnose_url
+from plinth.modules.apache.components import Webserver, diagnose_url
 from plinth.modules.backups.components import BackupRestore
 from plinth.modules.firewall.components import Firewall
 from plinth.modules.names.components import DomainType
@@ -21,7 +17,7 @@ from plinth.package import Packages
 from plinth.signals import domain_added, domain_removed
 from plinth.utils import format_lazy
 
-from . import manifest, utils
+from . import manifest, privileged, utils
 
 _description = [
     _('Tor is an anonymous communication system. You can learn more '
@@ -41,7 +37,7 @@ class TorApp(app_module.App):
 
     app_id = 'tor'
 
-    _version = 5
+    _version = 6
 
     def __init__(self):
         """Create components for the app."""
@@ -85,6 +81,10 @@ class TorApp(app_module.App):
                           (9040, 'tcp6'), (9053, 'udp4'), (9053, 'udp6')])
         self.add(daemon)
 
+        webserver = Webserver('webserver-onion-location',
+                              'onion-location-freedombox')
+        self.add(webserver)
+
         users_and_groups = UsersAndGroups('users-and-groups-tor',
                                           reserved_usernames=['debian-tor'])
         self.add(users_and_groups)
@@ -106,14 +106,26 @@ class TorApp(app_module.App):
                                          domain_type='domain-type-tor',
                                          name=hostname, services=services)
 
+    def enable(self):
+        """Enable the app and update firewall ports."""
+        super().enable()
+        privileged.update_ports()
+        update_hidden_service_domain()
+
+    def disable(self):
+        """Disable APT use of Tor before disabling."""
+        privileged.configure(apt_transport_tor=False)
+        super().disable()
+        update_hidden_service_domain()
+
     def diagnose(self):
         """Run diagnostics and return the results."""
         results = super().diagnose()
 
         results.extend(_diagnose_control_port())
 
-        output = actions.superuser_run('tor', ['get-status'])
-        ports = json.loads(output)['ports']
+        status = privileged.get_status()
+        ports = status['ports']
 
         results.append([
             _('Tor relay port available'),
@@ -156,15 +168,24 @@ class TorApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('tor',
-                              ['setup', '--old-version',
-                               str(old_version)])
+        privileged.setup(old_version)
         if not old_version:
-            actions.superuser_run(
-                'tor', ['configure', '--apt-transport-tor', 'enable'])
+            privileged.configure(apt_transport_tor=True)
 
-        update_hidden_service_domain()
-        self.enable()
+        update_hidden_service_domain(utils.get_status())
+
+        # Enable/disable Onion-Location component based on app status.
+        # Component was introduced in version 6.
+        if old_version and old_version < 6:
+            daemon_component = self.get_component('daemon-tor')
+            component = self.get_component('webserver-onion-location')
+            if daemon_component.is_enabled():
+                component.enable()
+            else:
+                component.disable()
+
+        if not old_version:
+            self.enable()
 
 
 def update_hidden_service_domain(status=None):
