@@ -34,12 +34,9 @@ logger = logging.getLogger(__name__)
 base_url = config['DEFAULT']['url']
 
 _app_checkbox_id = {
-    'tor': 'id_tor-enabled',
     'openvpn': 'id_openvpn-enabled',
 }
 
-_apps_with_loaders = ['tor']
-
 # unlisted sites just use '/' + site_name as url
 _site_url = {
     'wiki': '/ikiwiki',
@@ -47,13 +44,14 @@ _site_url = {
     'cockpit': '/_cockpit/',
     'syncthing': '/syncthing/',
     'rssbridge': '/rss-bridge/',
+    'ttrss': '/tt-rss/',
 }
 
 _sys_modules = [
     'avahi', 'backups', 'bind', 'cockpit', 'config', 'datetime', 'diagnostics',
     'dynamicdns', 'firewall', 'letsencrypt', 'names', 'networks', 'pagekite',
-    'performance', 'power', 'security', 'snapshot', 'ssh', 'storage',
-    'upgrades', 'users'
+    'performance', 'power', 'privacy', 'security', 'snapshot', 'ssh',
+    'storage', 'upgrades', 'users'
 ]
 
 
@@ -155,18 +153,20 @@ def get_password(username):
 
 
 def is_available(browser, site_name):
+    """Check if the given site_name is available."""
     url_to_visit = config['DEFAULT']['url'] + _get_site_url(site_name)
     browser.visit(url_to_visit)
     time.sleep(3)
     browser.reload()
-    not_404 = '404' not in browser.title
+    if '404' in browser.title or 'Page not found' in browser.title:
+        return False
+
     # The site might have a default path after the sitename,
     # e.g /mediawiki/Main_Page
     print('URL =', browser.url, url_to_visit, browser.title)
     browser_url = browser.url.partition('://')[2]
     url_to_visit_without_proto = url_to_visit.strip('/').partition('://')[2]
-    no_redirect = browser_url.startswith(url_to_visit_without_proto)
-    return not_404 and no_redirect
+    return browser_url.startswith(url_to_visit_without_proto)  # not a redirect
 
 
 def download_file(browser, url):
@@ -194,6 +194,10 @@ def download_file_outside_browser(url):
 # Form handling utilities #
 ###########################
 def submit(browser, element=None, form_class=None, expected_url=None):
+    """Submit a specific form in the current page and wait for page change."""
+    if not (element or form_class):
+        raise AssertionError('Either element or form_class must be sent')
+
     with wait_for_page_update(browser, expected_url=expected_url):
         if element:
             element.click()
@@ -226,9 +230,6 @@ def change_checkbox_status(browser, app_name, checkbox_id,
 
     submit(browser, form_class='form-configuration')
 
-    if app_name in _apps_with_loaders:
-        wait_for_config_update(browser, app_name)
-
 
 def wait_for_config_update(browser, app_name):
     """Wait until the configuration update progress goes away.
@@ -239,7 +240,7 @@ def wait_for_config_update(browser, app_name):
 
     """
     script = 'return (document.readyState == "complete") && ' \
-        '(!Boolean(document.querySelector(".running-status.loading")));'
+        '(!Boolean(document.querySelector(".app-operation")));'
     while not browser.execute_script(script):
         time.sleep(0.1)
 
@@ -257,7 +258,7 @@ def _create_admin_account(browser, username, password):
     browser.find_by_id('id_username').fill(username)
     browser.find_by_id('id_password1').fill(password)
     browser.find_by_id('id_password2').fill(password)
-    submit(browser)
+    submit(browser, form_class='form-create')
 
 
 def login(browser):
@@ -288,10 +289,10 @@ def login_with_account(browser, url, username, password=None):
         if login_button:
             browser.fill('username', username)
             browser.fill('password', password)
-            submit(browser)
+            submit(browser, form_class='form-login')
     else:
         browser.visit(base_url + '/plinth/firstboot/welcome')
-        submit(browser)  # click the "Start Setup" button
+        submit(browser, form_class='form-start')  # "Start Setup" button
         _create_admin_account(browser, username, password)
         if '/network-topology-first-boot' in browser.url:
             submit(browser, element=browser.find_by_name('skip')[0])
@@ -342,7 +343,8 @@ def install(browser, app_name):
     install_button_css = '.form-install input[type=submit]'
     while True:
         script = 'return (document.readyState == "complete") && ' \
-            '(!Boolean(document.querySelector(".app-operation")));'
+            '(!Boolean(document.querySelector(".app-operation"))) &&' \
+            '(!Boolean(document.querySelector(".app-just-installed")));'
         if not browser.execute_script(script):
             time.sleep(0.1)
         elif browser.is_element_present_by_css('.neterror'):
@@ -362,6 +364,34 @@ def install(browser, app_name):
             break
 
 
+def uninstall(browser, app_name):
+    """Uninstall the app if possible."""
+    nav_to_module(browser, app_name)
+    actions_button = browser.find_by_id('id_extra_actions_button')
+    if not actions_button:
+        pytest.skip('App cannot be uninstalled')
+
+    actions_button.click()
+    uninstall_item = browser.find_by_css('.uninstall-item')
+    if not uninstall_item:
+        pytest.skip('App cannot be uninstalled')
+
+    uninstall_item[0].click()
+    submit(browser, form_class='form-uninstall')
+
+    while True:
+        script = 'return (document.readyState == "complete") && ' \
+            '(!Boolean(document.querySelector(".app-operation")));'
+        if not browser.execute_script(script):
+            time.sleep(0.1)
+        elif browser.is_element_present_by_css('.neterror'):
+            browser.visit(browser.url)
+        elif browser.is_element_present_by_css('.alert-danger'):
+            raise RuntimeError('Uninstall failed')
+        else:
+            break
+
+
 ################################
 # App enable/disable utilities #
 ################################
@@ -380,9 +410,6 @@ def _change_app_status(browser, app_name, change_status_to='enabled'):
         change_checkbox_status(browser, app_name, checkbox_id,
                                change_status_to)
 
-    if app_name in _apps_with_loaders:
-        wait_for_config_update(browser, app_name)
-
 
 def app_enable(browser, app_name):
     nav_to_module(browser, app_name)
@@ -414,8 +441,7 @@ def app_can_be_disabled(browser, app_name):
 def set_domain_name(browser, domain_name):
     nav_to_module(browser, 'config')
     browser.find_by_id('id_domainname').fill(domain_name)
-    update_setup = browser.find_by_css('.btn-primary[type="submit"]')
-    submit(browser, element=update_setup)
+    submit(browser, form_class='form-configuration')
 
 
 ########################
@@ -463,6 +489,12 @@ def running_inside_container():
 ##############################
 # System -> Config utilities #
 ##############################
+def set_hostname(browser, hostname):
+    nav_to_module(browser, 'config')
+    browser.find_by_id('id_hostname').fill(hostname)
+    submit(browser, form_class='form-configuration')
+
+
 def set_advanced_mode(browser, mode):
     nav_to_module(browser, 'config')
     advanced_mode = browser.find_by_id('id_advanced_mode')
@@ -471,33 +503,34 @@ def set_advanced_mode(browser, mode):
     else:
         advanced_mode.uncheck()
 
-    update_setup = browser.find_by_css('.btn-primary')
-    submit(browser, element=update_setup)
+    submit(browser, form_class='form-configuration')
 
 
 ####################
 # Backup utilities #
 ####################
-def _click_button_and_confirm(browser, href):
+def _click_button_and_confirm(browser, href, form_class):
     buttons = browser.links.find_by_href(href)
     if buttons:
-        submit(browser, buttons.first)
-        submit(browser, expected_url='/plinth/sys/backups/')
+        submit(browser, element=buttons.first)
+        submit(browser, form_class=form_class,
+               expected_url='/plinth/sys/backups/')
 
 
 def _backup_delete_archive_by_name(browser, archive_name):
     nav_to_module(browser, 'backups')
     href = f'/plinth/sys/backups/root/delete/{archive_name}/'
-    _click_button_and_confirm(browser, href)
+    _click_button_and_confirm(browser, href, 'form-delete')
 
 
 def backup_create(browser, app_name, archive_name=None):
+    """Create a new backup for a given app."""
     install(browser, 'backups')
     if archive_name:
         _backup_delete_archive_by_name(browser, archive_name)
 
     buttons = browser.links.find_by_href('/plinth/sys/backups/create/')
-    submit(browser, buttons.first)
+    submit(browser, element=buttons.first)
     eventually(browser.find_by_css, args=['.select-all'])
     browser.find_by_css('.select-all').first.uncheck()
     if archive_name:
@@ -506,13 +539,14 @@ def backup_create(browser, app_name, archive_name=None):
     # ensure the checkbox is scrolled into view
     browser.execute_script('window.scrollTo(0, 0)')
     browser.find_by_value(app_name).first.check()
-    submit(browser)
+    submit(browser, form_class='form-backups')
 
 
 def backup_restore(browser, app_name, archive_name=None):
+    """Restore a given app from a backup archive."""
     nav_to_module(browser, 'backups')
     href = f'/plinth/sys/backups/root/restore-archive/{archive_name}/'
-    _click_button_and_confirm(browser, href)
+    _click_button_and_confirm(browser, href, 'form-restore')
 
 
 ######################
@@ -573,7 +607,7 @@ def create_user(browser, name, password=None, groups=[]):
     browser.find_by_id('id_confirm_password').fill(
         config['DEFAULT']['password'])
 
-    submit(browser)
+    submit(browser, form_class='form-create')
 
 
 def delete_user(browser, name):
@@ -584,7 +618,7 @@ def delete_user(browser, name):
 
     with wait_for_page_update(browser):
         delete_link.first.click()
-    submit(browser)
+    submit(browser, form_class='form-delete')
 
 
 def user_exists(browser, name):
@@ -607,6 +641,7 @@ class BaseAppTests:
     # TODO: Check the components of the app instead of configuring here.
     has_service = False
     has_web = True
+    can_uninstall = True
     check_diagnostics = True
     diagnostics_delay = 0
     disable_after_tests = True
@@ -669,3 +704,12 @@ class BaseAppTests:
         backup_create(session_browser, self.app_name, 'test_' + self.app_name)
         backup_restore(session_browser, self.app_name, 'test_' + self.app_name)
         self.assert_app_running(session_browser)
+
+    def test_uninstall(self, session_browser):
+        """Test that app can be uninstalled and installed back again."""
+        if not self.can_uninstall:
+            pytest.skip(f'Skipping uninstall test for {self.app_name}.')
+
+        uninstall(session_browser, self.app_name)
+        assert not is_installed(session_browser, self.app_name)
+        install(session_browser, self.app_name)
